@@ -1,4 +1,4 @@
-import functools
+import logging
 import platform
 import re
 
@@ -20,13 +20,31 @@ from .. import BaseProvider, ElementsType
 
 localized = True
 
+log = logging.getLogger(__name__)
+
 
 def datetime_to_timestamp(dt: Union[dtdate, datetime]) -> int:
-    if isinstance(dt, datetime) and getattr(dt, "tzinfo", None) is not None:
-        dt = dt.astimezone(tzutc())
+    if dt == datetime.min:
+        return timegm(dt.timetuple())
+    if isinstance(dt, datetime):
+        try:
+            dt = dt.astimezone(tzutc())
+        except OSError:
+            pass
     return timegm(dt.timetuple())
 
 
+def convert_timestamp_to_datetime(timestamp: Union[int, float], tzinfo: TzInfo) -> datetime:
+    if timestamp >= 0:
+        try:
+            return datetime.fromtimestamp(timestamp, tzinfo)
+        except OSError:
+            log.exception("OSError occurred while converting timestamp '%s' to datetime", timestamp)
+            raise
+    else:
+        return datetime(1970, 1, 1, tzinfo=tzinfo) + timedelta(seconds=int(timestamp))
+
+
 def timestamp_to_datetime(timestamp: Union[int, float], tzinfo: Optional[TzInfo]) -> datetime:
     if tzinfo is None:
         pick = convert_timestamp_to_datetime(timestamp, tzlocal())
@@ -34,20 +52,6 @@ def timestamp_to_datetime(timestamp: Union[int, float], tzinfo: Optional[TzInfo]
     return convert_timestamp_to_datetime(timestamp, tzinfo)
 
 
-def maybe_format_as_string(func):
-    """Format date as string if a pattern is provided."""
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        result: dtdate = func(*args, **kwargs)
-        pattern = kwargs.get("pattern")
-        if pattern:
-            return result.strftime(pattern)
-        return result
-
-    return wrapper
-
-
 def change_year(current_date: dtdate, year_diff: int) -> dtdate:
     """
     Unless the current_date is February 29th, it is fine to just subtract years.
@@ -1832,6 +1836,137 @@ class Provider(BaseProvider):
 
     regex = re.compile(timedelta_pattern)
 
+    @classmethod
+    def _is_absolute(cls, obj: Optional[DateParseType]) -> bool:
+        if obj is None:
+            return False
+        if isinstance(obj, (datetime, dtdate, int)):
+            return True
+        elif isinstance(obj, timedelta):
+            return False
+        elif isinstance(obj, str):
+            if obj in ("today", "now"):
+                return False
+            return cls.regex.fullmatch(obj) is None
+        return False
+
+    @classmethod
+    def _get_reference_date_time(
+        cls, start_date: Optional[DateParseType], end_date: Optional[DateParseType], tzinfo: Optional[TzInfo]
+    ) -> datetime:
+        """
+        Return Which datetime is absolute, or now if both are relative.
+        If both are absolute, return the most recent one.
+        If both are None, return now.
+        """
+
+        min_ = datetime_to_timestamp(datetime.min)
+        now = datetime.now(tzinfo)
+        if start_date is None and end_date is None:
+            return now
+        if start_date is None and end_date is not None:
+            start_date = now
+        elif start_date is not None and end_date is None:
+            end_date = now
+
+        start_int = cls._parse_date_time(start_date, now) if start_date is not None else min_
+        end_int = cls._parse_date_time(end_date, now) if end_date is not None else min_
+        if not cls._is_absolute(start_date) and not cls._is_absolute(end_date):
+            return now
+        if cls._is_absolute(start_date) and cls._is_absolute(end_date):
+            reference = max([start_int, end_int])
+        elif cls._is_absolute(start_date) and not cls._is_absolute(end_date):
+            reference = start_int
+        elif cls._is_absolute(end_date) and not cls._is_absolute(start_date):
+            reference = end_int
+        return timestamp_to_datetime(reference, tzinfo)
+
+    @classmethod
+    def _get_reference_date(cls, start_date: Optional[DateParseType], end_date: Optional[DateParseType]) -> dtdate:
+        reference = cls._get_reference_date_time(start_date, end_date, None)
+        return reference.date()
+
+    @classmethod
+    def _parse_start_datetime(cls, now: datetime, value: Optional[DateParseType]) -> int:
+        if value is None:
+            return 0
+
+        return cls._parse_date_time(value, now)
+
+    @classmethod
+    def _parse_end_datetime(cls, now: datetime, value: Optional[DateParseType]) -> int:
+        if value is None:
+            return datetime_to_timestamp(now)
+
+        return cls._parse_date_time(value, now)
+
+    @classmethod
+    def _parse_date_string(cls, value: str) -> Dict[str, float]:
+        parts = cls.regex.match(value)
+        if not parts:
+            raise ParseError(f"Can't parse date string `{value}`")
+        parts = parts.groupdict()
+        time_params: Dict[str, float] = {}
+        for name_, param_ in parts.items():
+            if param_:
+                time_params[name_] = int(param_)
+
+        if "years" in time_params:
+            if "days" not in time_params:
+                time_params["days"] = 0
+            time_params["days"] += 365.24 * time_params.pop("years")
+        if "months" in time_params:
+            if "days" not in time_params:
+                time_params["days"] = 0
+            time_params["days"] += 30.42 * time_params.pop("months")
+
+        if not time_params:
+            raise ParseError(f"Can't parse date string `{value}`")
+        return time_params
+
+    @classmethod
+    def _parse_timedelta(cls, value: Union[timedelta, str, float]) -> Union[float, int]:
+        if isinstance(value, timedelta):
+            return value.total_seconds()
+        if isinstance(value, str):
+            time_params = cls._parse_date_string(value)
+            return timedelta(**time_params).total_seconds()  # type: ignore
+        if isinstance(value, (int, float)):
+            return value
+        raise ParseError(f"Invalid format for timedelta {value!r}")
+
+    @classmethod
+    def _parse_date_time(cls, value: DateParseType, now: datetime, tzinfo: Optional[TzInfo] = None) -> int:
+        if isinstance(value, (datetime, dtdate)):
+            return datetime_to_timestamp(value)
+        if isinstance(value, timedelta):
+            return datetime_to_timestamp(now + value)
+        if isinstance(value, str):
+            if value == "now":
+                return datetime_to_timestamp(datetime.now(tzinfo))
+            time_params = cls._parse_date_string(value)
+            return datetime_to_timestamp(now + timedelta(**time_params))  # type: ignore
+        if isinstance(value, int):
+            return value
+        raise ParseError(f"Invalid format for date {value!r}")
+
+    @classmethod
+    def _parse_date(cls, value: DateParseType, today: dtdate) -> dtdate:
+        if isinstance(value, datetime):
+            return value.date()
+        elif isinstance(value, dtdate):
+            return value
+        if isinstance(value, timedelta):
+            return today + value
+        if isinstance(value, str):
+            if value in ("today", "now"):
+                return today
+            time_params = cls._parse_date_string(value)
+            return today + timedelta(**time_params)  # type: ignore
+        if isinstance(value, int):
+            return today + timedelta(value)
+        raise ParseError(f"Invalid format for date {value!r}")
+
     def unix_time(
         self,
         end_datetime: Optional[DateParseType] = None,
@@ -1845,17 +1980,25 @@ class Provider(BaseProvider):
 
         :example: 1061306726.6
         """
-        start_datetime = self._parse_start_datetime(start_datetime)
-        end_datetime = self._parse_end_datetime(end_datetime)
+        if start_datetime is not None and end_datetime is None:
+            if start_datetime == "now":
+                end_datetime = "+30d"
+            else:
+                end_datetime = datetime.now(tz=tzutc())
+        elif start_datetime is None and end_datetime is not None:
+            start_datetime = datetime(1970, 1, 1, tzinfo=tzutc())
+        now = self._get_reference_date_time(start_datetime, end_datetime, tzinfo=tzutc())
+        start_datetime = self._parse_start_datetime(now, start_datetime)
+        end_datetime = self._parse_end_datetime(now, end_datetime)
         return float(self._rand_seconds(start_datetime, end_datetime))
 
     def time_delta(self, end_datetime: Optional[DateParseType] = None) -> timedelta:
         """
         Get a timedelta object
         """
-        start_datetime = self._parse_start_datetime("now")
-        end_datetime = self._parse_end_datetime(end_datetime)
-        seconds = end_datetime - start_datetime
+        now = datetime.now()
+        end = self._parse_end_datetime(now, end_datetime)
+        seconds = end - datetime_to_timestamp(now)
 
         ts = self._rand_seconds(*sorted([0, seconds]))
         return timedelta(seconds=ts)
@@ -1897,8 +2040,9 @@ class Provider(BaseProvider):
         # simply change that class method to use this magic number as a
         # default value when None is provided.
 
-        start_time = -62135596800 if start_datetime is None else self._parse_start_datetime(start_datetime)
-        end_datetime = self._parse_end_datetime(end_datetime)
+        now = self._get_reference_date_time(start_datetime, end_datetime, tzinfo)
+        start_time = -62135596800 if start_datetime is None else self._parse_start_datetime(now, start_datetime)
+        end_datetime = self._parse_end_datetime(now, end_datetime)
 
         ts = self._rand_seconds(start_time, end_datetime)
         # NOTE: using datetime.fromtimestamp(ts) directly will raise
@@ -1962,89 +2106,6 @@ class Provider(BaseProvider):
         """
         return self.date_time(end_datetime=end_datetime).time()
 
-    @classmethod
-    def _parse_start_datetime(cls, value: Optional[DateParseType]) -> int:
-        if value is None:
-            return 0
-
-        return cls._parse_date_time(value)
-
-    @classmethod
-    def _parse_end_datetime(cls, value: Optional[DateParseType]) -> int:
-        if value is None:
-            return datetime_to_timestamp(datetime.now())
-
-        return cls._parse_date_time(value)
-
-    @classmethod
-    def _parse_date_string(cls, value: str) -> Dict[str, float]:
-        parts = cls.regex.match(value)
-        if not parts:
-            raise ParseError(f"Can't parse date string `{value}`")
-        parts = parts.groupdict()
-        time_params: Dict[str, float] = {}
-        for name_, param_ in parts.items():
-            if param_:
-                time_params[name_] = int(param_)
-
-        if "years" in time_params:
-            if "days" not in time_params:
-                time_params["days"] = 0
-            time_params["days"] += 365.24 * time_params.pop("years")
-        if "months" in time_params:
-            if "days" not in time_params:
-                time_params["days"] = 0
-            time_params["days"] += 30.42 * time_params.pop("months")
-
-        if not time_params:
-            raise ParseError(f"Can't parse date string `{value}`")
-        return time_params
-
-    @classmethod
-    def _parse_timedelta(cls, value: Union[timedelta, str, float]) -> Union[float, int]:
-        if isinstance(value, timedelta):
-            return value.total_seconds()
-        if isinstance(value, str):
-            time_params = cls._parse_date_string(value)
-            return timedelta(**time_params).total_seconds()  # type: ignore
-        if isinstance(value, (int, float)):
-            return value
-        raise ParseError(f"Invalid format for timedelta {value!r}")
-
-    @classmethod
-    def _parse_date_time(cls, value: DateParseType, tzinfo: Optional[TzInfo] = None) -> int:
-        if isinstance(value, (datetime, dtdate)):
-            return datetime_to_timestamp(value)
-        now = datetime.now(tzinfo)
-        if isinstance(value, timedelta):
-            return datetime_to_timestamp(now + value)
-        if isinstance(value, str):
-            if value == "now":
-                return datetime_to_timestamp(datetime.now(tzinfo))
-            time_params = cls._parse_date_string(value)
-            return datetime_to_timestamp(now + timedelta(**time_params))  # type: ignore
-        if isinstance(value, int):
-            return value
-        raise ParseError(f"Invalid format for date {value!r}")
-
-    @classmethod
-    def _parse_date(cls, value: DateParseType) -> dtdate:
-        if isinstance(value, datetime):
-            return value.date()
-        elif isinstance(value, dtdate):
-            return value
-        today = dtdate.today()
-        if isinstance(value, timedelta):
-            return today + value
-        if isinstance(value, str):
-            if value in ("today", "now"):
-                return today
-            time_params = cls._parse_date_string(value)
-            return today + timedelta(**time_params)  # type: ignore
-        if isinstance(value, int):
-            return today + timedelta(value)
-        raise ParseError(f"Invalid format for date {value!r}")
-
     def date_time_between(
         self,
         start_date: DateParseType = "-30y",
@@ -2061,8 +2122,12 @@ class Provider(BaseProvider):
         :example: datetime('1999-02-02 11:42:52')
         :return: datetime
         """
-        start_date = self._parse_date_time(start_date, tzinfo=tzinfo)
-        end_date = self._parse_date_time(end_date, tzinfo=tzinfo)
+        if end_date is None:
+            end_date = "now"
+
+        now = self._get_reference_date_time(start_date, end_date, tzinfo)
+        start_date = self._parse_date_time(start_date, now, tzinfo=tzinfo)
+        end_date = self._parse_date_time(end_date, now, tzinfo=tzinfo)
         if end_date - start_date <= 1:
             ts = start_date + self.generator.random.random()
         else:
@@ -2072,26 +2137,22 @@ class Provider(BaseProvider):
         else:
             return (datetime(1970, 1, 1, tzinfo=tzutc()) + timedelta(seconds=ts)).astimezone(tzinfo)
 
-    @maybe_format_as_string
-    def date_between(
-        self,
-        start_date: DateParseType = "-30y",
-        end_date: DateParseType = "today",
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def date_between(self, start_date: DateParseType = "-30y", end_date: DateParseType = "today") -> dtdate:
         """
         Get a Date object based on a random date between two given dates.
         Accepts date strings that can be recognized by strtotime().
 
         :param start_date: Defaults to 30 years ago
         :param end_date: Defaults to "today"
-        :param pattern: optional pattern to format the date as string
-        :example: Date('1999-02-02') or '1999-02-02'
-        :return: Date or string
+        :example: Date('1999-02-02')
+        :return: Date
         """
 
-        start_date = self._parse_date(start_date)
-        end_date = self._parse_date(end_date)
+        if end_date is None:
+            end_date = "now"
+        today = self._get_reference_date(start_date, end_date)
+        start_date = self._parse_date(start_date, today)
+        end_date = self._parse_date(end_date, today)
         return self.date_between_dates(date_start=start_date, date_end=end_date)
 
     def future_datetime(self, end_date: DateParseType = "+30d", tzinfo: Optional[TzInfo] = None) -> datetime:
@@ -2107,13 +2168,7 @@ class Provider(BaseProvider):
         """
         return self.date_time_between(start_date="+1s", end_date=end_date, tzinfo=tzinfo)
 
-    @maybe_format_as_string
-    def future_date(
-        self,
-        end_date: DateParseType = "+30d",
-        tzinfo: Optional[TzInfo] = None,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def future_date(self, end_date: DateParseType = "+30d", tzinfo: Optional[TzInfo] = None) -> dtdate:
         """
         Get a Date object based on a random date between 1 day from now and a
         given date.
@@ -2121,7 +2176,6 @@ class Provider(BaseProvider):
 
         :param end_date: Defaults to "+30d"
         :param tzinfo: timezone, instance of datetime.tzinfo subclass
-        :param pattern: optional pattern to format the date as string
         :example: dtdate('2030-01-01')
         :return: dtdate
         """
@@ -2140,13 +2194,7 @@ class Provider(BaseProvider):
         """
         return self.date_time_between(start_date=start_date, end_date="-1s", tzinfo=tzinfo)
 
-    @maybe_format_as_string
-    def past_date(
-        self,
-        start_date: DateParseType = "-30d",
-        tzinfo: Optional[TzInfo] = None,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def past_date(self, start_date: DateParseType = "-30d", tzinfo: Optional[TzInfo] = None) -> dtdate:
         """
         Get a Date object based on a random date between a given date and 1 day
         ago.
@@ -2154,9 +2202,8 @@ class Provider(BaseProvider):
 
         :param start_date: Defaults to "-30d"
         :param tzinfo: timezone, instance of datetime.tzinfo subclass
-        :param pattern: optional pattern to format the date as string
         :example: dtdate('1999-02-02')
-        :return: dtdate or string
+        :return: dtdate
         """
         return self.date_between(start_date=start_date, end_date="-1d")
 
@@ -2177,13 +2224,17 @@ class Provider(BaseProvider):
         :example: datetime('1999-02-02 11:42:52')
         :return: datetime
         """
+        today = self._get_reference_date(datetime_start, datetime_end)
+        now = datetime.combine(today, datetime.min.time(), tzinfo)
         datetime_start_ = (
             datetime_to_timestamp(datetime.now(tzinfo))
             if datetime_start is None
-            else self._parse_date_time(datetime_start)
+            else self._parse_date_time(datetime_start, now)
         )
         datetime_end_ = (
-            datetime_to_timestamp(datetime.now(tzinfo)) if datetime_end is None else self._parse_date_time(datetime_end)
+            datetime_to_timestamp(datetime.now(tzinfo))
+            if datetime_end is None
+            else self._parse_date_time(datetime_end, now)
         )
 
         timestamp = self._rand_seconds(datetime_start_, datetime_end_)
@@ -2191,7 +2242,7 @@ class Provider(BaseProvider):
             if tzinfo is None:
                 pick = convert_timestamp_to_datetime(timestamp, tzlocal())
                 try:
-                    pick = pick.astimezone(tzutc()).replace(tzinfo=None)
+                    pick = pick.replace(tzinfo=None)
                 except OSError:
                     pass
             else:
@@ -2203,21 +2254,18 @@ class Provider(BaseProvider):
             )
         return pick
 
-    @maybe_format_as_string
     def date_between_dates(
         self,
         date_start: Optional[DateParseType] = None,
         date_end: Optional[DateParseType] = None,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    ) -> dtdate:
         """
         Takes two Date objects and returns a random date between the two given dates.
         Accepts Date or datetime objects
 
         :param date_start: Date
         :param date_end: Date
-        :param pattern: optional pattern to format the date as string
-        :return: Date or string
+        :return: Date
         """
         return self.date_time_between_dates(date_start, date_end).date()
 
@@ -2333,21 +2381,14 @@ class Provider(BaseProvider):
         else:
             return now
 
-    @maybe_format_as_string
-    def date_this_century(
-        self,
-        before_today: bool = True,
-        after_today: bool = False,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def date_this_century(self, before_today: bool = True, after_today: bool = False) -> dtdate:
         """
         Gets a Date object for the current century.
 
         :param before_today: include days in current century before today
         :param after_today: include days in current century after today
-        :param pattern: optional pattern to format the date as string
         :example: Date('2012-04-04')
-        :return: Date or string
+        :return: Date
         """
         today = dtdate.today()
         this_century_start = dtdate(today.year - (today.year % 100), 1, 1)
@@ -2362,21 +2403,14 @@ class Provider(BaseProvider):
         else:
             return today
 
-    @maybe_format_as_string
-    def date_this_decade(
-        self,
-        before_today: bool = True,
-        after_today: bool = False,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def date_this_decade(self, before_today: bool = True, after_today: bool = False) -> dtdate:
         """
         Gets a Date object for the decade year.
 
         :param before_today: include days in current decade before today
         :param after_today: include days in current decade after today
-        :param pattern: optional pattern to format the date as string
         :example: Date('2012-04-04')
-        :return: Date or string
+        :return: Date
         """
         today = dtdate.today()
         this_decade_start = dtdate(today.year - (today.year % 10), 1, 1)
@@ -2391,19 +2425,12 @@ class Provider(BaseProvider):
         else:
             return today
 
-    @maybe_format_as_string
-    def date_this_year(
-        self,
-        before_today: bool = True,
-        after_today: bool = False,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def date_this_year(self, before_today: bool = True, after_today: bool = False) -> dtdate:
         """
         Gets a Date object for the current year.
 
         :param before_today: include days in current year before today
         :param after_today: include days in current year after today
-        :param pattern: optional pattern to format the date as string
         :example: Date('2012-04-04')
         :return: Date
         """
@@ -2420,21 +2447,14 @@ class Provider(BaseProvider):
         else:
             return today
 
-    @maybe_format_as_string
-    def date_this_month(
-        self,
-        before_today: bool = True,
-        after_today: bool = False,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    def date_this_month(self, before_today: bool = True, after_today: bool = False) -> dtdate:
         """
         Gets a Date object for the current month.
 
         :param before_today: include days in current month before today
         :param after_today: include days in current month after today
-        :param pattern: optional pattern to format the date as string
         :example: dtdate('2012-04-04')
-        :return: dtdate or string
+        :return: dtdate
         """
         today = dtdate.today()
         this_month_start = today.replace(day=1)
@@ -2465,8 +2485,11 @@ class Provider(BaseProvider):
         ``distrib`` is a callable that accepts ``<datetime>`` and returns ``<value>``
 
         """
-        start_date_ = self._parse_date_time(start_date, tzinfo=tzinfo)
-        end_date_ = self._parse_date_time(end_date, tzinfo=tzinfo)
+        if end_date is None:
+            end_date = "now"
+        now = self._get_reference_date_time(start_date, end_date, tzinfo)
+        start_date_ = self._parse_date_time(start_date, now, tzinfo=tzinfo)
+        end_date_ = self._parse_date_time(end_date, now, tzinfo=tzinfo)
 
         if end_date_ < start_date_:
             raise ValueError("`end_date` must be greater than `start_date`.")
@@ -2524,14 +2547,12 @@ class Provider(BaseProvider):
         """
         return gettz(self.timezone(*args, **kwargs))  # type: ignore
 
-    @maybe_format_as_string
     def date_of_birth(
         self,
         tzinfo: Optional[TzInfo] = None,
         minimum_age: int = 0,
         maximum_age: int = 115,
-        pattern: Optional[str] = None,
-    ) -> Union[dtdate, str]:
+    ) -> dtdate:
         """
         Generate a random date of birth represented as a Date object,
         constrained by optional miminimum_age and maximum_age
@@ -2540,7 +2561,6 @@ class Provider(BaseProvider):
         :param tzinfo: Defaults to None.
         :param minimum_age: Defaults to 0.
         :param maximum_age: Defaults to 115.
-        :param pattern: optional pattern to format the date as string
 
         :example: Date('1979-02-02')
         :return: Date
@@ -2572,12 +2592,3 @@ class Provider(BaseProvider):
         dob = self.date_time_ad(tzinfo=tzinfo, start_datetime=start_date, end_datetime=end_date).date()
 
         return dob if dob != start_date else dob + timedelta(days=1)
-
-
-def convert_timestamp_to_datetime(timestamp: Union[int, float], tzinfo: TzInfo) -> datetime:
-    import datetime as dt
-
-    if timestamp >= 0:
-        return dt.datetime.fromtimestamp(timestamp, tzinfo)
-    else:
-        return dt.datetime(1970, 1, 1, tzinfo=tzinfo) + dt.timedelta(seconds=int(timestamp))
